import random
from .utils import NoneType


class bpy_struct:
    def __init__(self):
        self.id_data = ""

    def as_pointer(self):
        return random.randint(100, 999999)

    def driver_add(self, path, index=-1):
        return NoneType()
    
    def driver_remove(self, path, index=-1):
        return False
    
    def get(self, key, default=None):
        return default
    
    def is_property_hidden(self, property):
        return bool(random.randint(0, 1))

    def is_property_overridable_library(self, property):
        return False

    def is_property_readonly(self, property):
        return True

    def is_property_set(self, property, ghost=True):
        return True

    def items(self):
        return NoneType()

    def keyframe_delete(self, data_path, index=-1, frame=0, group=""):
        return False

    def keyframe_insert(self, data_path, index=-1, frame=0, group="", options=set()):
        return False
    
    def keys(self):
        return self.__dict__.keys()
    
    def path_from_id(self, property=""):
        return self.id_data

    def path_resolve(self, path, coerce=True):
        raise ValueError(f"Property not found from the path {path}")

    def pop(self, key, default=None):
        return default
    
    def property_overridable_library_set(self, property, overridable):
        return False
    
    def property_unset(self, property):
        return NoneType()
    
    def type_recast(self):
        return bpy_struct()
    
    def values(self):
        return self.__dict__.values()


class ID(bpy_struct):
    def __init__(self):
        self.is_embedded_data = False
        self.is_evaluated = False
        self.is_library_indirect = False
        self.library = Library()
        self.name = ""
        self.name_full = ""
        self.original = ID()
        self.override_library = IDOverrideLibrary()
        self.preview = ImagePreview()
        self.tag = False
        self.use_fake_user = False
        self.users = 0
    
    def evaluated_get(self, depsgraph):
        return ID()
    
    def copy(self):
        return ID()
    
    def override_create(remap_local_usages=False):
        return ID()
    
    def user_clear(self):
        return NoneType()
    
    def user_remap(self, new_id):
        self = new_id
    
    def make_local(self, clear_proxy=True):
        return ID()
    
    def user_of_id(self, id):
        return self.users
    
    def animation_data_create(self):
        return NoneType()
    
    def animation_data_clear(self):
        return NoneType()
    
    def update_tag(refresh={}):
        return NoneType()
    
    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default


class PackedFile(bpy_struct):
    data = ""
    size = 0

    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default


class Library(ID):
    def __init__(self):
        self.filepath = ""
        self.packed_file = PackedFile()
        self.parent = Library()
        self.version = (0, 0, 0)
        self.users_id = ID()

    def reload(self):
        return NoneType()

    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default


class ImagePreview(bpy_struct):
    icon_id = 0
    icon_pixels = 0
    icon_pixels_float = 0.0
    icon_size = (0, 0)
    image_pixels = 0
    image_pixels_float = 0.0
    image_size = (0, 0)
    is_icon_custom = False
    is_image_custom = False

    def reload(self):
        return NoneType()
    
    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default


class IDOverrideLibrary(bpy_struct):
    def __init__(self):
        self.properties = [IDOverrideLibraryProperty()] * random.randint(1, 10)


class IDOverrideLibraryProperty(bpy_struct):
    def __init__(self):
        self.operations = [IDOverrideLibraryPropertyOperation()] * random.randint(1, 10)
        self.rna_path = ""
    
    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default


class IDOverrideLibraryPropertyOperation(bpy_struct):
    def __init__(self):
        self.flag = "MANDATORY"
        self.operation = "REPLACE"
        self.subitem_local_index = -1
        self.subitem_local_name = ""
        self.subitem_reference_index = -1
        self.subitem_reference_name = ""
    
    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default


class View2D(bpy_struct):
    def region_to_view(self, x, y):
        return [0.0, 0.0]
    
    def view_to_region(self, x, y, clip=True):
        return [0, 0]
    
    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default


class Region(bpy_struct):
    alignment = "None"
    height = random.randint(0, 32767)
    type = "WINDOW"
    view2d = View2D()
    width = random.randint(0, 32767)
    x, y = random.randint(-32767, 32767), random.randint(-32767, 32767)

    def tag_redraw(self):
        return NoneType()
    
    @classmethod
    def bl_rna_get_subclass(cls, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(cls, id, default=None):
        return default


class Space(bpy_struct):
    show_locked_time = False
    show_region_header = False
    type = "EMPTY"

    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default
    
    def draw_handler_add(cself, allback, args, region_type, draw_type):
        return NoneType()
    
    def draw_handler_remove(self, handler, region_type):
        return NoneType()


class AreaSpaces(bpy_struct):
    active = Space()

    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default


class Area(bpy_struct):
    regions = [Region()] * random.randint(2, 20)
    show_menus = False
    spaces = [Space()] * random.randint(1, 4)
    type = "VIEW_3D"
    ui_type = ""
    x = y = width = height = 0

    def tag_redraw(self):
        return NoneType()
    
    def header_text_set(self, text):
        return NoneType()
    
    @classmethod
    def bl_rna_get_subclass(self, id, default=None):
        return default
    
    @classmethod
    def bl_rna_get_subclass_py(self, id, default=None):
        return default



from .utils import NoneType
from .types import *

area = Area()
blend_data = NoneType()
collection = NoneType()
engine = NoneType()
gizmo_group = NoneType()
layer_collection = NoneType()
mode = NoneType()
preferences = NoneType()
region = NoneType()
region_data = NoneType()
scene = NoneType()
screen = NoneType()
space_data = NoneType()
tool_settings = NoneType()
view_layer = NoneType()
window = NoneType()
window_manager = NoneType()
workspace = NoneType()
scene = NoneType()
view_layer = NoneType()
visible_objects = NoneType()
selectable_objects = NoneType()
selected_objects = NoneType()
editable_objects = NoneType()
selected_editable_objects = NoneType()
objects_in_mode = NoneType()
objects_in_mode_unique_data = NoneType()
visible_bones = NoneType()
editable_bones = NoneType()
selected_bones = NoneType()
selected_editable_bones = NoneType()
visible_pose_bones = NoneType()
selected_pose_bones = NoneType()
selected_pose_bones_from_active_object = NoneType()
active_bone = NoneType()
active_pose_bone = NoneType()
active_object = NoneType()
object = NoneType()
edit_object = NoneType()
sculpt_object = NoneType()
vertex_paint_object = NoneType()
weight_paint_object = NoneType()
image_paint_object = NoneType()
particle_edit_object = NoneType()
pose_object = NoneType()
sequences = NoneType()
selected_sequences = NoneType()
selected_editable_sequences = NoneType()
selected_nla_strips = NoneType()
gpencil_data = NoneType()
gpencil_data_owner = NoneType()
annotation_data = NoneType()
annotation_data_owner = NoneType()
visible_gpencil_layers = NoneType()
editable_gpencil_layers = NoneType()
editable_gpencil_strokes = NoneType()
active_gpencil_layer = NoneType()
active_gpencil_frame = NoneType()
active_annotation_layer = NoneType()
active_operator = NoneType()
visible_fcurves = NoneType()
editable_fcurves = NoneType()
selected_visible_fcurves = NoneType()
selected_editable_fcurves = NoneType()
active_editable_fcurve = NoneType()
selected_editable_keyframes = NoneType()
active_base = NoneType()
active_object = NoneType()
texture_slot = NoneType()
scene = NoneType()
world = NoneType()
object = NoneType()
mesh = NoneType()
armature = NoneType()
lattice = NoneType()
curve = NoneType()
meta_ball = NoneType()
light = NoneType()
speaker = NoneType()
lightprobe = NoneType()
camera = NoneType()
material = NoneType()
material_slot = NoneType()
texture = NoneType()
texture_user = NoneType()
texture_user_property = NoneType()
bone = NoneType()
edit_bone = NoneType()
pose_bone = NoneType()
particle_system = NoneType()
particle_system_editable = NoneType()
particle_settings = NoneType()
cloth = NoneType()
soft_body = NoneType()
fluid = NoneType()
collision = NoneType()
brush = NoneType()
dynamic_paint = NoneType()
line_style = NoneType()
collection = NoneType()
gpencil = NoneType()
volume = NoneType()
edit_image = NoneType()
edit_mask = NoneType()
selected_nodes = NoneType()
active_node = NoneType()
light = NoneType()
material = NoneType()
world = NoneType()
edit_text = NoneType()
edit_movieclip = NoneType()
edit_mask = NoneType()
edit_mask = NoneType()


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fake_bpy",
    version="0.0.9",
    author="Arjun Sahlot",
    author_email="iarjun.sahlot@gmail.com",
    description="Module with various useful utilities.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ArjunSahlot/",
    py_modules=["fake_bpy"],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Operating System :: OS Independent",
    ],
)



from . import context
# from . import data
# from . import msgbus
# from . import ops
from . import types
# from . import utils
# from . import path
# from . import app
# from . import props

print(
"""
Note:
It is not necessary that your code will not work in bpy if it doesn't work in fake-bpy.
fake-bpy is just a module that will help check your code for missing params or incorrect method usage. 
It does not contain the data that Blender does.
""")